@@ -40,6 +40,7 @@
 #include "SPIRVInstruction.h"
 #include "SPIRVBasicBlock.h"
 #include "SPIRVFunction.h"
+#include "SPIRVInternal.h"
 
 #include <unordered_set>
 
@@ -157,6 +158,35 @@ std::vector<SPIRVType *> SPIRVInstruction::getOperandTypes() {
   return getOperandTypes(getOperands());
 }
 
+void SPIRVImageInstBase::setOpWords(const std::vector<SPIRVWord> &OpsArg) {
+  std::vector<SPIRVWord> Ops = OpsArg;
+
+  // If the Image Operands field has the SignExtend or ZeroExtend bit set,
+  // either raise the minimum SPIR-V version to 1.4, or drop the operand
+  // if SPIR-V 1.4 cannot be emitted.
+  size_t ImgOpsIndex = getImageOperandsIndex(OpCode);
+  if (ImgOpsIndex != ~0U && ImgOpsIndex < Ops.size()) {
+    SPIRVWord ImgOps = Ops[ImgOpsIndex];
+    unsigned SignZeroExtMasks = ImageOperandsMask::ImageOperandsSignExtendMask |
+                                ImageOperandsMask::ImageOperandsZeroExtendMask;
+    if (ImgOps & SignZeroExtMasks) {
+      SPIRVModule *M = getModule();
+      if (M->isAllowedToUseVersion(VersionNumber::SPIRV_1_4)) {
+        M->setMinSPIRVVersion(static_cast<SPIRVWord>(VersionNumber::SPIRV_1_4));
+      } else {
+        // Drop SignExtend/ZeroExtend if we cannot use SPIR-V 1.4.
+        Ops[ImgOpsIndex] &= ~SignZeroExtMasks;
+        if (Ops[ImgOpsIndex] == 0) {
+          // Drop the Image Operands if SignExtend/ZeroExtend was the only
+          // bit set.
+          Ops.pop_back();
+        }
+      }
+    }
+  }
+  SPIRVInstTemplateBase::setOpWords(Ops);
+}
+
 bool isSpecConstantOpAllowedOp(Op OC) {
   static SPIRVWord Table[] = {
       OpSConvert,
